﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Internal.Mapping;
using ArcGIS.Desktop.Internal.Mapping.Locate;
using ArcGIS.Desktop.Mapping;

namespace GeoNet259853
{
	internal class SelectionCoincidence : Button
	{
		private string waterServiceName = @"GISWRKS1.WORKS.WaterService";
		private string waterFittingsName = @"GISWRKS1.WORKS.WAT_Fitting";

		protected override async void OnClick()
		{
			var isOk = false;
			try
			{
				var waterServiceLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(waterServiceName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
				if (waterServiceLayer != null)
				{
					var waterFittingsLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(waterFittingsName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
					if (waterFittingsLayer != null)
					{
						isOk = await QueuedTask.Run<bool>(() =>
						{
							bool bResult = false;
							var selFittings = waterFittingsLayer.GetSelection();
							if (selFittings.GetCount() == 0) return bResult;
							var selService = waterServiceLayer.GetSelection();
							if (selService.GetCount() == 0) return bResult;
							var curFitting = waterFittingsLayer.Search(new QueryFilter() { ObjectIDs = selFittings.GetObjectIDs() });
							var curService = waterServiceLayer.Search(new QueryFilter() { ObjectIDs = selService.GetObjectIDs() });
							while (curFitting.MoveNext())
							{
								using (Feature featFitting = curFitting.Current as Feature)
								{
									while (curService.MoveNext())
									{
										using (Feature featService = curService.Current as Feature)
										{
											Polyline lineService = featService.GetShape() as Polyline;
											if (lineService == null) throw new Exception("Selected Service is not a valid line");
											MapPoint pntFitting = featFitting.GetShape() as MapPoint;
											if (pntFitting == null) throw new Exception("Selected fitting is not a valid pnt");
											MapPoint toPoint = lineService.Points[lineService.PointCount - 1];
											MessageBox.Show($@"Coincident: {toPoint.IsEqual(pntFitting)} {Environment.NewLine}End point [{toPoint.X} - {toPoint.Y}] {Environment.NewLine}Plug point [{pntFitting.X} - {pntFitting.Y}]");
											bResult = true;
										}
									}
								}
							}
							return bResult;
						});
					}
				}
				if (!isOk) MessageBox.Show($@"Nothing checked - select line endpoint and coindicent points");
			}
			catch (Exception ex)
			{
				MessageBox.Show($@"Error: {ex}");
			}
		}
	}
}
